unit Unit4;

interface

uses
  SysUtils, Inienums, CustomTransformer,
  StringTransformer, Contnrs, TetraTypes, Ini, Classes;

type

  TDataModule4 = class(TDataModule)
    StringTransformer1: TStringTransformer;
    procedure StringTransformer1EnterProduction(Sender: TObject;
      const xasName: string; xeSymno: Integer);
    procedure StringTransformer1ExitProduction(Sender: TObject);
    procedure DataModuleCreate(Sender: TObject);
    procedure DataModuleDestroy(Sender: TObject);
    procedure StringTransformer1Exception(Sender: TObject; xasWhat: string);
    procedure StringTransformer1ParserMessage(Sender: TObject; xasWhat: string;
      xlwLastPosition, xlwPosition: Cardinal; const xasProduction,
      xasSymbol: string; xeMessageType: eParserMessage; xlw: Cardinal);
    procedure StringTransformer1AcceptToken(Sender: TObject;
      const xasName: string; xeSymno: Integer; xlwIgnoredBegin,
      xlwRecognizedBegin, xlwRecognizedEnd: Cardinal; xbIsSkip: Boolean);
  private
    Input : AnsiString;
    Stack : TStack;
    Ini : TIni;
    LastSection : TSection;
    LastEntry : TEntry;

    function GetProject() : AnsiString;
    function GetInputFile() : AnsiString;
    function GetTop() : Integer;
    function GetLeft() : Integer;
    function GetWidth() : Integer;
    function GetHeight() : Integer;
    procedure SetProject(xs : AnsiString);
    procedure SetInputFile(xs : AnsiString);
    procedure SetTop(xi : Integer);
    procedure SetLeft(xi : Integer);
    procedure SetWidth(xi : Integer);
    procedure SetHeight(xi : Integer);

    function Recognized(xlwRecognizedBegin, xlwRecognizedEnd: Cardinal) : AnsiString;
  public
    procedure Parse(const xasInput : AnsiString );
    procedure WriteIni();
  published
  property Project : AnsiString read GetProject write SetProject;
  property InputFile : AnsiString read GetInputFile write SetInputFile;
  property Top : Integer read GetTop write SetTop;
  property Left : Integer read GetLeft write SetLeft;
  property Width : Integer read GetWidth write SetWidth;
  property Height : Integer read GetHeight write SetHeight;
  end;

var
  DataModule4: TDataModule4;

implementation

{$R *.dfm}

uses Forms, StrUtils;


procedure TDataModule4.DataModuleCreate(Sender: TObject);
begin
  Stack := TStack.Create;
  Ini := TIni.Create;
end;

procedure TDataModule4.DataModuleDestroy(Sender: TObject);
begin
  Stack.Free();
  FreeAndNil(Ini);
end;

procedure TDataModule4.Parse(const xasInput : AnsiString );
begin
  Input := xasInput;
  StringTransformer1.TransformToTree(xasInput);
end;

function TDataModule4.Recognized(xlwRecognizedBegin, xlwRecognizedEnd: Cardinal) : AnsiString;
begin
  Result := MidStr(Input, xlwRecognizedBegin, xlwRecognizedEnd - xlwRecognizedBegin);
end;

procedure TDataModule4.StringTransformer1AcceptToken(Sender: TObject;
  const xasName: string; xeSymno: Integer; xlwIgnoredBegin, xlwRecognizedBegin,
  xlwRecognizedEnd: Cardinal; xbIsSkip: Boolean);
var
  e : EProduction;
begin
  e := EProduction(Stack.Peek());
  case EToken(xeSymno) of
  tkID :
  begin
    case e of
    pdSection:
    begin
      LastSection := Ini.NewSection( Recognized( xlwRecognizedBegin, xlwRecognizedEnd));
    end;
    pdEntry:
    begin
      if LastSection <> NIL then
         LastEntry := LastSection.NewEntry( Recognized( xlwRecognizedBegin, xlwRecognizedEnd));
    end;
    end;
  end;
  tkROL:
  begin
    if LastEntry <> NIL  then
    begin
       LastEntry.Value :=
           Recognized( xlwRecognizedBegin, xlwRecognizedEnd);
    end;
  end;
  end;
end;

procedure TDataModule4.StringTransformer1EnterProduction(Sender: TObject;
  const xasName: string; xeSymno: Integer);
begin
  Stack.Push(TObject(xeSymno));
end;

procedure TDataModule4.StringTransformer1Exception(Sender: TObject;
  xasWhat: string);
begin
  Application.MessageBox(PChar(xasWhat), 'Exception');
end;

procedure TDataModule4.StringTransformer1ExitProduction(Sender: TObject);
var
  e : EProduction;
begin
  e := EProduction(Stack.Peek());
  case e of
  pdSection:
  LastSection := NIL;
  pdEntry:
  LastEntry := NIL;
  end;

  Stack.Pop();


end;

procedure TDataModule4.StringTransformer1ParserMessage(Sender: TObject;
  xasWhat: string; xlwLastPosition, xlwPosition: Cardinal; const xasProduction,
  xasSymbol: string; xeMessageType: eParserMessage; xlw: Cardinal);
begin
  Application.MessageBox(PChar(xasWhat), 'ParserMessage');

end;

function TDataModule4.GetProject() : AnsiString;
begin
  Result := '';
  if Ini <> NIL then
    Result := Ini.GetValue('FILES', 'PROJECT', 'Text2Html.ttp');
end;

procedure TDataModule4.SetProject(xs : AnsiString);
begin
  if Ini <> NIL then
     Ini.SetValue('FILES', 'PROJECT', xs);
end;


function TDataModule4.GetInputFile() : AnsiString;
begin
  Result := '';
  if Ini <> NIL then
    Result := Ini.GetValue('FILES', 'INPUT', '.\\Text2Html_sources\\Help.txt');
end;

procedure TDataModule4.SetInputFile(xs : AnsiString);
begin
  if Ini <> NIL then
     Ini.SetValue('FILES', 'INPUT', xs);
end;


function TDataModule4.GetTop() : Integer;
begin
  Result := 0;
  if Ini <> NIL then
    Result := StrToInt(Ini.GetValue('LAYOUT', 'TOP', '90'));
end;

procedure TDataModule4.SetTop(xi : Integer);
begin
  if Ini <> NIL then
     Ini.SetValue('LAYOUT', 'TOP', IntToStr(xi));
end;


function TDataModule4.GetLeft() : Integer;
begin
  Result := 0;
  if Ini <> NIL then
    Result := StrToInt(Ini.GetValue('LAYOUT', 'LEFT', '170'));
end;

procedure TDataModule4.SetLeft(xi : Integer);
begin
  if Ini <> NIL then
     Ini.SetValue('LAYOUT', 'LEFT', IntToStr(xi));
end;


function TDataModule4.GetWidth() : Integer;
begin
  Result := 0;
  if Ini <> NIL then
    Result := StrToInt(Ini.GetValue('LAYOUT', 'WIDTH', '590'));
end;

procedure TDataModule4.SetWidth(xi : Integer);
begin
  if Ini <> NIL then
     Ini.SetValue('LAYOUT', 'WIDTH', IntToStr(xi));
end;


function TDataModule4.GetHeight() : Integer;
begin
  Result := 0;
  if Ini <> NIL then
    Result := StrToInt(Ini.GetValue('LAYOUT', 'HEIGHT', '390'));
end;

procedure TDataModule4.SetHeight(xi : Integer);
begin
  if Ini = NIL then EXIT;
  Ini.SetValue('LAYOUT', 'HEIGHT', IntToStr(xi));
end;

procedure TDataModule4.WriteIni();
begin
  if Ini <> NIL then
     Ini.WriteIni();
end;

end.
