unit TetraDll; // extracted from TetraComponents;
//----------------------------------------------------------
//   TetraComponents
//   TextTransformer Support
//   Author: Dr. Detlef Meyer-Eltz
//   Many thanks to Dr. Hans-Peter Diettrich for his PASCAL advices
//   http://www.texttransformer.de
//   http://www.texttransformer.com
//   Meyer-Eltz@t-online.de
//
//   March, 2008
//----------------------------------------------------------

interface

uses
  TetraTypes;

type

  FnOnBeginDocument = procedure (const xpName : PAnsiChar); cdecl;
  FnOnEnterProduction = procedure (const xpName : PAnsiChar; xeSymno : INTEGER); cdecl;
  FnOnExit = procedure (); cdecl;
  FnOnBranch = procedure (const xpName : PAnsiChar; xeKind : eTetraNodeKind); cdecl;
  FnOnAcceptToken = procedure (const xpName : PAnsiChar;  xeSymno : INTEGER;
    xlwIgnoredBegin, xlwRecognizedBegin, xlwRecognizedEnd : longword;
    xbIsSkip : Boolean); cdecl;
  FnOnException = procedure (const xpWhat : PAnsiChar); cdecl;
  FnOnSourceTarget = procedure (const xpSourceFile, xpTargetFile, xpMessage : PAnsiChar); cdecl;
  FnOnTTMessage = procedure (const xpWhat : PAnsiChar;
    xlwLastPosition, xlwPosition : longword;
    const xpProduction, xpSymbol : PAnsiChar; xeMessageType : eParserMessage;
    xlw : longword = 0); cdecl;
  FnContinue = function () : BOOLEAN; cdecl;


  INIT = procedure (); cdecl;
  FREE = procedure (); cdecl;
  GETVERSIONMAJOR = function () : INTEGER; cdecl;
  GETVERSIONMINOR = function () : INTEGER; cdecl;
  GETVERSIONRELEASE = function () : INTEGER; cdecl;
  LOADPROJECT = function (const xpFilename : PAnsiChar) : BOOLEAN; cdecl;
  COMPILEPROJECT = function (): BOOLEAN; cdecl;
  SETSOURCEROOT  = procedure (const xpSourceRoot : PAnsiChar);  cdecl;
  SETTARGETROOT  = procedure (const xpTargetRoot : PAnsiChar);  cdecl;
  SETCONFIGPARAM  = procedure (const xpConfigParam : PAnsiChar);  cdecl;
  //ADDSYMBOL  = procedure (const xp : PAnsiChar);  cdecl;
  TRANSFORMTOFILEN2ONE  = function (const xpSource, xpTarget : PAnsiChar;
    xbFirstFile : BOOLEAN; xbLastFile : BOOLEAN; const xpExtraParam : PAnsiChar) : BOOLEAN; cdecl;
  TRANSFORMTOFILESONE2ONE  = function (const xpSource, xpTarget, xpExtraParam : PAnsiChar) : BOOLEAN; cdecl;
  BACKUPTRANSFORM  = function (const xpSource, xpBackup, xpTarget, xpExtraParam : PAnsiChar) : BOOLEAN; cdecl;
  TRANSFORMTEXTTOTEXT  = function (const xpText, xpExtraParam : PAnsiChar) : PAnsiChar; cdecl;
  TRANSFORMTEXTTOTREE  = function (const xpText, xpExtraParam : PAnsiChar) : BOOLEAN; cdecl;
  SETONBEGINDOCUMENT  = procedure (xpOnBeginDocument : FnOnBeginDocument); cdecl;
  SETONENDDOCUMENT  = procedure (xpOnExit : FnOnExit); cdecl;
  SETONENTERPRODUCTION  = procedure (xpOnEnter : FnOnEnterProduction); cdecl;
  SETONEXITPRODUCTION  = procedure (xpOnExit : FnOnExit); cdecl;
  SETONACCEPTTOKEN = procedure (xpOnAcceptToken : FnOnAcceptToken); cdecl;
  SETONBEGINBRANCH  = procedure (xpOnEnter : FnOnBranch); cdecl;
  SETONENDBRANCH  = procedure (xpOnExit : FnOnExit); cdecl;
  SETONPARSERMESSAGE  = procedure (xpOnMessage : FnOnTTMessage); cdecl;
  SETONEXCEPTION  = procedure (xpOnException : FnOnException); cdecl;
  SETONFILEERROR = procedure (xpOnError : FnOnSourceTarget); cdecl;
  SETCONTINUEFN = procedure (xpContinue : FnContinue); cdecl;
  ENABLEACTIONS = procedure (xbEnable : BOOLEAN); cdecl;

  TTetra_Dll = class
  private
    m_hEngine: HMODULE;
    m_iVersionMajor : INTEGER;
    m_iVersionMinor : INTEGER;
    m_iVersionRelease : INTEGER;
    m_bActivationError: BOOLEAN;
  public
    _Init : INIT;
    _Free : FREE;
    _GetVersionMajor : GETVERSIONMAJOR;
    _GetVersionMinor : GETVERSIONMINOR;
    _GetVersionRelease : GETVERSIONRELEASE;
    _LoadProject : LOADPROJECT;
    _CompileProject : COMPILEPROJECT;
    _SetSourceRoot : SETSOURCEROOT;
    _SetTargetRoot : SETTARGETROOT;
    _SetConfigParam : SETCONFIGPARAM;
    _TransformToFileN2One : TRANSFORMTOFILEN2ONE;
    _TransformToFileOne2One : TRANSFORMTOFILESONE2ONE;
    _BackupTransform : BACKUPTRANSFORM;
    _TransformTextToText : TRANSFORMTEXTTOTEXT;
    _TransformTextToTree : TRANSFORMTEXTTOTREE;
    _SetOnBeginDocument : SETONBEGINDOCUMENT;
    _SetOnEndDocument : SETONENDDOCUMENT;
    _SetOnEnterProduction : SETONENTERPRODUCTION;
    _SetOnExitProduction : SETONEXITPRODUCTION;
    _SetOnAcceptToken : SETONACCEPTTOKEN;
    _SetOnBeginBranch : SETONBEGINBRANCH;
    _SetOnEndBranch : SETONENDBRANCH;
    _SetOnParserMessage : SETONPARSERMESSAGE;
    _SetOnException : SETONEXCEPTION;
    _SetOnFileError : SETONFILEERROR;
    _SetContinueFn : SETCONTINUEFN;
    _EnableActions : ENABLEACTIONS;

    constructor Create();
    destructor Destroy; override;

    function IsDllLoaded() : BOOLEAN;
    function HasLoadError() : BOOLEAN;
    procedure FreeDll();
    procedure FreeLibrary();
    function Init() : BOOLEAN;

    //function CheckVersion() : BOOLEAN;
    function GetComponentVersionString() : AnsiString;
    function GetDLLVersionString() : AnsiString;
    function LoadEngine() : BOOLEAN;
    procedure EnableActions(xbEnable : BOOLEAN);
    function SetActive(): boolean;  //return succ
    function TransformToFileN2One(xpSource : PAnsiChar;
                                  const xpTarget : PAnsiChar;
                                  xbFirstFile : BOOLEAN;
                                  xbLastFile : BOOLEAN;
                                  const xpExtraParam : PAnsiChar) : BOOLEAN;
    function TransformToFileOne2One(const xpSource : PAnsiChar;
                                    const xpTarget : PAnsiChar;
                                    const xpExtraParam : PAnsiChar) : BOOLEAN;
    function BackupTransform(const xpSource: PAnsiChar;
                             const xpBackup : PAnsiChar;
                             const xpTarget : PAnsiChar;
                             const xpExtraParam : PAnsiChar) : BOOLEAN;

  end;

var
  _TetraDll : TTetra_Dll;
//---------------------------------------------------------------------------

implementation

uses
  Windows, SysUtils;

{ TTetra_Dll }

constructor TTetra_Dll.Create();
begin
  m_iVersionMajor := 1;
  m_iVersionMinor := 7;
  m_iVersionRelease := 2;

end;

destructor TTetra_Dll.Destroy;
begin
   FreeLibrary();
   inherited Destroy;
end;

function TTetra_Dll.IsDllLoaded() : BOOLEAN;
begin
  if HasLoadError then
  begin
    result := false;
    EXIT;
  end;

  result := (m_hEngine <> 0) and
            Assigned(_Free) and
            Assigned(_Init) and
            Assigned(_LoadProject) and
            Assigned(_GetVersionMajor) and
            Assigned(_GetVersionMinor) and
            Assigned(_GetVersionRelease) and
            Assigned(_CompileProject) and
            Assigned(_SetSourceRoot) and
            Assigned(_SetTargetRoot) and
            Assigned(_SetConfigParam) and
            Assigned(_TransformToFileN2One) and
            Assigned(_TransformToFileOne2One) and
            Assigned(_BackupTransform) and
            Assigned(_TransformTextToText) and
            Assigned(_TransformTextToTree) and
            Assigned(_SetOnBeginDocument) and
            Assigned(_SetOnEndDocument) and
            Assigned(_SetOnEnterProduction) and
            Assigned(_SetOnExitProduction) and
            Assigned(_SetOnAcceptToken) and
            Assigned(_SetOnBeginBranch) and
            Assigned(_SetOnEndBranch) and
            Assigned(_SetOnParserMessage) and
            Assigned(_SetOnException) and
            Assigned(_SetOnFileError) and
            Assigned(_SetContinueFn) and
            Assigned(_EnableActions);
end;

procedure TTetra_Dll.FreeDll();
begin
  if m_hEngine <> 0 then
  begin
    _Free();
    _GetVersionMajor := NIL;
    _GetVersionMinor := NIL;
    _GetVersionRelease := NIL;
    _LoadProject := NIL;
    _CompileProject := NIL;
    _SetSourceRoot := NIL;
    _SetTargetRoot := NIL;
    _SetConfigParam := NIL;
    _TransformToFileN2One := NIL;
    _TransformToFileOne2One := NIL;
    _BackupTransform := NIL;
    _TransformTextToText := NIL;
    _TransformTextToTree := NIL;
    _SetOnBeginDocument := NIL;
    _SetOnEndDocument := NIL;
    _SetOnEnterProduction := NIL;
    _SetOnExitProduction := NIL;
    _SetOnAcceptToken := NIL;
    _SetOnBeginBranch := NIL;
    _SetOnEndBranch := NIL;
    _SetOnParserMessage := NIL;
    _SetOnException := NIL;
    _SetOnFileError := NIL;
    _SetContinueFn := NIL;
    _EnableActions := NIL;
  end;
end;
//---------------------------------------------------------------------------
procedure TTetra_Dll.FreeLibrary();
begin
  if m_hEngine <> 0 then
  begin
    FreeDll();
    Windows.FreeLibrary(m_hEngine);
    m_hEngine := 0;
  end
end;
//---------------------------------------------------------------------------
function TTetra_Dll.Init() : BOOLEAN;
begin
  Result := m_hEngine <> 0;
  if not Result then
    exit;
  _Init := GetProcAddress(m_hEngine, '_Init');
  _Free := GetProcAddress(m_hEngine, '_Free');
  _GetVersionMajor := GetProcAddress(m_hEngine, '_GetVersionMajor');
  _GetVersionMinor := GetProcAddress(m_hEngine, '_GetVersionMinor');
  _GetVersionRelease := GetProcAddress(m_hEngine, '_GetVersionRelease');
  _LoadProject := GetProcAddress(m_hEngine, '_LoadProject');
  _CompileProject := GetProcAddress(m_hEngine, '_CompileProject');
  _SetSourceRoot := GetProcAddress(m_hEngine, '_SetSourceRoot');
  _SetTargetRoot := GetProcAddress(m_hEngine, '_SetTargetRoot');
  _SetConfigParam := GetProcAddress(m_hEngine, '_SetConfigParam');
  _TransformToFileN2One := GetProcAddress(m_hEngine, '_TransformToFileN2One');
  _TransformToFileOne2One := GetProcAddress(m_hEngine, '_TransformToFileOne2One');
  _BackupTransform := GetProcAddress(m_hEngine, '_BackupTransform');
  _TransformTextToText := GetProcAddress(m_hEngine, '_TransformTextToText');
  _TransformTextToTree := GetProcAddress(m_hEngine, '_TransformTextToTree');
  _SetOnBeginDocument := GetProcAddress(m_hEngine, '_SetOnBeginDocument');
  _SetOnEndDocument := GetProcAddress(m_hEngine, '_SetOnEndDocument');
  _SetOnEnterProduction := GetProcAddress(m_hEngine, '_SetOnEnterProduction');
  _SetOnExitProduction := GetProcAddress(m_hEngine, '_SetOnExitProduction');
  _SetOnAcceptToken := GetProcAddress(m_hEngine, '_SetOnAcceptToken');
  _SetOnBeginBranch := GetProcAddress(m_hEngine, '_SetOnBeginBranch');
  _SetOnEndBranch := GetProcAddress(m_hEngine, '_SetOnEndBranch');
  _SetOnParserMessage := GetProcAddress(m_hEngine, '_SetOnParserMessage');
  _SetOnException := GetProcAddress(m_hEngine, '_SetOnException');
  _SetOnFileError := GetProcAddress(m_hEngine, '_SetOnFileError');
  _SetContinueFn := GetProcAddress(m_hEngine, '_SetContinueFn');
  _EnableActions := GetProcAddress(m_hEngine, '_EnableActions');

  Result := IsDllLoaded();
  if Result then
    _Init();
end;
//---------------------------------------------------------------------------
function TTetra_Dll.HasLoadError() : BOOLEAN;
begin
  result := m_bActivationError;
end;

//---------------------------------------------------------------------------
function TTetra_Dll.LoadEngine() : BOOLEAN;
begin
  Result := not m_bActivationError;
  if not Result then
    EXIT;

  Result := IsDllLoaded();
  if Result then
    exit;

  if m_hEngine = 0 then
    m_hEngine := LoadLibrary('tetra_engine.dll');

  Result := Init();
  m_bActivationError := not Result;
end;
//---------------------------------------------------------------------------
procedure TTetra_Dll.EnableActions(xbEnable : BOOLEAN);
begin
  _EnableActions(xbEnable);
end;

(*
//---------------------------------------------------------------------------
function TTetra_Dll.CheckVersion() : BOOLEAN;
begin
  Result := not m_bActivationError;
  if not Result then
    EXIT;

  Result := (_GetVersionMajor() = m_iVersionMajor) and
            (_GetVersionMinor() = m_iVersionMinor) and
            (_GetVersionRelease() = m_iVersionRelease)
end;
*)
//---------------------------------------------------------------------------
function TTetra_Dll.GetComponentVersionString() : AnsiString;
begin
  Result := IntToStr(m_iVersionMajor) + '.' +
            IntToStr(m_iVersionMinor) + '.' +
            IntToStr(m_iVersionRelease)

end;
//---------------------------------------------------------------------------
function TTetra_Dll.GetDllVersionString() : AnsiString;
begin
  if IsDllLoaded then
    Result := IntToStr(_GetVersionMajor()) + '.' +
              IntToStr(_GetVersionMinor()) + '.' +
              IntToStr(_GetVersionRelease())
  else
   Result := '0.0.0';
end;
//---------------------------------------------------------------------------
function TTetra_Dll.SetActive(): boolean;
begin
  if not IsDllLoaded and (m_hEngine <> 0) then
     Init();
  Result := IsDllLoaded;
end;
//---------------------------------------------------------------------------
function TTetra_Dll.TransformToFileN2One(xpSource : PAnsiChar;
                                         const xpTarget : PAnsiChar;
                                         xbFirstFile : BOOLEAN;
                                         xbLastFile : BOOLEAN;
                                         const xpExtraParam : PAnsiChar) : BOOLEAN;
begin
  result := _TransformToFileN2One(xpSource,
                                  xpTarget,
                                  xbFirstFile,
                                  xbLastFile,
                                  xpExtraParam);
end;
//---------------------------------------------------------------------------
function TTetra_Dll.TransformToFileOne2One(const xpSource : PAnsiChar;
                                           const xpTarget : PAnsiChar;
                                           const xpExtraParam : PAnsiChar) : BOOLEAN;
begin
  result := _TransformToFileOne2One(xpSource, xpTarget, xpExtraParam);
end;
//---------------------------------------------------------------------------
function TTetra_Dll.BackupTransform(const xpSource: PAnsiChar;
                                    const xpBackup : PAnsiChar;
                                    const xpTarget : PAnsiChar;
                                    const xpExtraParam : PAnsiChar) : BOOLEAN;
begin
  result := _BackupTransform(xpSource, xpBackup, xpTarget, xpExtraParam);
end;

//---------------------------------------------------------------------------
initialization
  _TetraDll := TTetra_Dll.Create;
//---------------------------------------------------------------------------
finalization
   _TetraDll.Free;
end.
