unit TetraTree;

interface

uses
  Classes,
  ComCtrls, //TTreeView
  StdCtrls,
  CustomTransformer,
  TetraDll,
  TetraTypes,
  TetraMessage;

type

  eTreeType = (
    ttTokenlist,   // = 0;
    ttRelationTree,      // = 1;
    ttProductionTree,   // = 2;
    ttFullTree   // = 3;
  );

  TTetraNode = class(TTreeNode)
  protected
    SymbolNumber : INTEGER;
    Kind: eTetraNodeKind;
    SourceText: AnsiString;  //should be property of TTreeNodes (Owner.Source)
    IgnoredBegin,
    RecognizedBegin,
    RecognizedEnd : Cardinal;
  public
    function  IgnoredText: AnsiString;
    function  RecognizedText: AnsiString;
    function  Symno: Integer;
  end;

  TTetraNodeClass = class of TTetraNode;

  PTetraNodeLink = ^TTetraNodeLink;
  TTetraNodeLink = record
    m_This : TTetraNode;
    m_pFirstChild : PTetraNodeLink;
    m_pNextSibling : PTetraNodeLink;
//  not allowed in C6  procedure addChildLast( xpNode : PTetraNodeLink);
  end;
 

  TBeforeNewNode = procedure(Sender: TCustomTextTransformer; const xasName: AnsiString; Kind: eTetraNodeKind) of object;
  TAfterNewNode = procedure(Node: TTetraNode) of object;

  TTextTreeViewTransformer = class (TCustomTextTransformer)
  private
    m_Tree: TTreeView;
    m_Memo: TCustomMemo;
    m_eTreetype : eTreeType;
    m_nHelpProduction : TTetraNode;  // parent for new nodes
    m_pnlHelpProduction : PTetraNodeLink;  // parent for new nodes in relation tree
    m_ProductionList : TList;  // helps to construct a relation tree
    m_pnlReturn : PTetraNodeLink;  // helps to construct a relation tree

    procedure InsertChildren(node : PTetraNodeLink);

  protected
    FOnBeforeNewNode: TBeforeNewNode;
    FOnAfterNewNode: TAfterNewNode;
    procedure BeforeAcceptToken(const xasName: string; xeSymno : INTEGER; xuiIgnoredBegin, xuiRecognizedBegin, xuiRecognizedEnd: Cardinal; xbIsSkip: Boolean); override;
    procedure BeforeEnterProduction(const xasName: string; xeSymno : INTEGER); override;
    procedure BeforeExitProduction(); override;
    procedure BeforeBeginBranch(const xasName: string; xeNodeKind : eTetraNodeKind); override;
    procedure BeforeEndBranch(); override;

    function CreateNode(kind: eTetraNodeKind): TTetraNode;
    function CreateNodeLink(kind: eTetraNodeKind): PTetraNodeLink;
    function AddNode(kind: eTetraNodeKind; const xasName: string): TTetraNode;

 //   procedure Notification(AComponent: TComponent; Operation: TOperation); override;
    function CheckMemoExists(): BOOLEAN;
    procedure SetMemo(const Value : TCustomMemo);

  public
    constructor Create(Owner : TComponent); override;
    function  Execute(const xasExtraParam: AnsiString = '') : BOOLEAN; override;
    procedure SetTree(xTree: TTreeView );
    procedure SetTreeType(xeTreeType: eTreeType );
    procedure Clear();

  published
    property EnableActions : BOOLEAN read m_bEnableActions write SetEnableActions;
    property OnEnterProduction : TOnEnterProduction read FOnEnterProduction  write FOnEnterProduction;
    property OnExitProduction : TOnExit read FOnExitProduction  write FOnExitProduction;
    property OnAcceptToken : TOnAcceptToken read FOnAcceptToken  write FOnAcceptToken;
    property OnBeginBranch : TOnBranch read FOnBeginBranch  write FOnBeginBranch;
    property OnEndBranch : TOnExit read FOnEndBranch  write FOnEndBranch;
    property Tree : TTreeView read m_Tree write SetTree;
    property Input : TCustomMemo read m_Memo write SetMemo;
    property TreeType : eTreeType read m_eTreeType write SetTreeType;
    property OnBeforeNewNode: TBeforeNewNode read FOnBeforeNewNode write FOnBeforeNewNode;
    property OnAfterNewNode: TAfterNewNode read FOnAfterNewNode write FOnAfterNewNode;
  end;
  //---------------------------------------------------------------------------

implementation

uses
  StrUtils;


procedure AddChildLast( xpParent, xpChild : PTetraNodeLink);
var
  pChild : PTetraNodeLink;
begin
 if xpParent.m_pFirstChild = NIL then begin
    xpParent.m_pFirstChild := xpChild;
    xpChild.m_pNextSibling := NIL;
  end
  else
  begin
    pChild := xpParent.m_pFirstChild;
    while  pChild.m_pNextSibling <> NIL do
      pChild := pChild.m_pNextSibling;

    //now pChild is last child
    pChild.m_pNextSibling := xpChild;
    xpChild.m_pNextSibling := NIL;
  end;
end;

{ TTextTreeViewTransformer }

constructor TTextTreeViewTransformer.Create(Owner : TComponent);
begin
  inherited Create(Owner);
  m_Tree := NIL;
  m_Memo := NIL;
  m_bEnableActions := false;
end;

procedure TTextTreeViewTransformer.Clear();
begin
  if Assigned(m_Tree) then
     m_Tree.Items.Clear();
end;
//---------------------------------------------------------------------------
procedure TTextTreeViewTransformer.SetTree(xTree : TTreeView );
begin
  if Assigned(m_Tree) then
    m_Tree.RemoveFreeNotification(Self);

  m_Tree := xTree;
  SetModified(true);
  m_nHelpProduction := NIL;

  if Assigned(m_Tree) then
    m_Tree.FreeNotification(Self)
   else
    Stop();

end;
//---------------------------------------------------------------------------
procedure TTextTreeViewTransformer.SetTreeType(xeTreeType: eTreeType );
begin
  m_eTreeType := xeTreeType;
  SetModified(true);
  m_nHelpProduction := NIL;
end;
//---------------------------------------------------------------------------
procedure TTextTreeViewTransformer.SetMemo(const Value : TCustomMemo);
begin
  if Assigned(m_Memo) then
    m_Memo.RemoveFreeNotification(Self);

  m_Memo := Value;

  if Assigned(m_Memo) then
    m_Memo.FreeNotification(Self)
   else
    Stop();

end;
//---------------------------------------------------------------------------
function TTextTreeViewTransformer.CheckMemoExists(): BOOLEAN;
begin
  result := true;
  if not Assigned(m_Memo) then
     result := false;
end;
(*  BDS 2007 hangs when closing a project with this function
//---------------------------------------------------------------------------
procedure TTextTreeViewTransformer.Notification(AComponent : TComponent;
  Operation : TOperation);
begin
  if Operation = opRemove then begin
    if AComponent = m_Memo then
       m_Memo := Nil
    else
    if AComponent = m_Tree then
       m_Tree := Nil;
  end;
end;  *)
//---------------------------------------------------------------------------
function TTextTreeViewTransformer.Execute(const xasExtraParam: AnsiString): BOOLEAN;
begin
  if (Tree <> nil) and CheckMemoExists() then begin //and (Modified or (m_sExtra <> xasExtraParam)) then begin
    if SetActive() then begin
      try
      Clear;
      m_nHelpProduction := nil;
      m_ProductionList := TList.Create();
      m_pnlHelpProduction := NIL;
      m_pnlReturn := NIL;
      ClearMessages();
      _TetraDll._TransformTextToTree(PAnsiChar(m_Memo.Lines.Text), PAnsiChar(xasExtraParam));
      finally
         m_ProductionList.Free();
      end;
    end;  //else OnException or OnParserMessage
  end;
  SetModified(false);
  result := not HasError();
end;

function TTextTreeViewTransformer.CreateNode(kind: eTetraNodeKind): TTetraNode;
begin
  Result := TTetraNode.Create(m_Tree.Items);
  Result.Kind := kind;
  if CheckMemoExists() then
     Result.SourceText := m_Memo.Lines.Text;
end;

function TTextTreeViewTransformer.CreateNodeLink(kind: eTetraNodeKind): PTetraNodeLink;
var
  p : PTetraNodeLink;
begin
  New(p);
  Result := p;
  p^.m_This := CreateNode(kind);
  p^.m_pFirstChild := NIL;
  p^.m_pNextSibling := NIL;
end;

function TTextTreeViewTransformer.AddNode(kind: eTetraNodeKind;
  const xasName: string): TTetraNode;
begin
  Result := CreateNode(kind);
  m_Tree.Items.AddNode(Result, m_nHelpProduction, xasName, nil, naAddChild);
end;

//---------------------------------------------------------------------------

procedure TTextTreeViewTransformer.BeforeAcceptToken(
  const xasName: string; xeSymno : INTEGER; xuiIgnoredBegin, xuiRecognizedBegin,
  xuiRecognizedEnd: Cardinal; xbIsSkip: Boolean);
var
  n: TTetraNode;
  p: PTetraNodeLink;
const
  pPos = nil;
  //TokenKind: array[boolean] of eTetraNodeKind = (tnkToken, tnkSkip);
begin
(** add new node as last child of m_nHelpProduction
*)
  if m_Tree = NIL then
    EXIT;

  case m_eTreeType of
  ttProductionTree, ttFullTree, ttTokenList :
  begin
    if xbIsSkip then
      n := AddNode(tnkSkip, xasName)
     else
      n := AddNode(tnkToken, xasName);
    n.SymbolNumber := xeSymno;
    n.IgnoredBegin := xuiIgnoredBegin;
    n.RecognizedBegin := xuiRecognizedBegin;
    n.RecognizedEnd := xuiRecognizedEnd;
  end;

  ttRelationTree :
  begin
    if m_pnlReturn <> NIL then
    begin
      AddChildLast(m_pnlHelpProduction, m_pnlReturn);
      m_pnlReturn := NIL;
    end;

    if m_pnlHelpProduction.m_pFirstChild <> NIL then
    begin
      if xbIsSkip then
         p := CreateNodeLink(tnkSkip)
       else
         p := CreateNodeLink(tnkToken);
      p.m_This.SymbolNumber := xeSymno;
      p.m_This.IgnoredBegin := xuiIgnoredBegin;
      p.m_This.RecognizedBegin := xuiRecognizedBegin;
      p.m_This.RecognizedEnd := xuiRecognizedEnd;
      p.m_This.Text := xasName;
      AddChildLast(m_pnlHelpProduction, p);
    end
     else
     begin
      if xbIsSkip then
         m_pnlReturn := CreateNodeLink(tnkSkip)
       else
         m_pnlReturn := CreateNodeLink(tnkToken);
      m_pnlReturn.m_This.SymbolNumber := xeSymno;
      m_pnlReturn.m_This.IgnoredBegin := xuiIgnoredBegin;
      m_pnlReturn.m_This.RecognizedBegin := xuiRecognizedBegin;
      m_pnlReturn.m_This.RecognizedEnd := xuiRecognizedEnd;
      m_pnlReturn.m_This.Text := xasName;
     end;
  end;
  end;
end;

procedure TTextTreeViewTransformer.BeforeEnterProduction(const xasName: string; xeSymno : INTEGER);
begin
(** add new node as last child of m_nHelpProduction (root node if nil).
  Make the new node the new m_nHelpProduction.
*)
  if m_Tree = NIL then
     EXIT;

  case m_eTreeType of
  ttProductionTree, ttFullTree :
  begin
    m_nHelpProduction := AddNode(tnkProduction, xasName);
  end;
  ttRelationTree :
  begin
    if m_pnlReturn <> NIL then
    begin
      AddChildLast(m_pnlHelpProduction, m_pnlReturn);
      m_pnlReturn := NIL;
    end;
    m_pnlHelpProduction := CreateNodeLink(tnkProduction);
    m_pnlHelpProduction.m_This.Text := xasName;
    m_ProductionList.Add(m_pnlHelpProduction);
  end;
  end;
end;

procedure TTextTreeViewTransformer.BeforeExitProduction();
var
  n : TTetraNode;
  p: PTetraNodeLink;
  pnReturn : PTetraNodeLink;
begin
(** Revert m_nHelpProduction to its parent.
  (all child nodes have been added)

  What if no children? The positions stay zero, may invalidate the parent's positions!
*)
  if m_Tree = NIL then
     EXIT;

  case m_eTreeType of
  ttProductionTree, ttFullTree :
  begin
    n := m_nHelpProduction.GetLastChild as TTetraNode;
    if n <> nil then begin
    //has children, get the total range
      m_nHelpProduction.RecognizedEnd := n.RecognizedEnd;
      n := m_nHelpProduction.GetFirstChild as TTetraNode;
      m_nHelpProduction.SymbolNumber := n.Symno;
      m_nHelpProduction.IgnoredBegin := n.IgnoredBegin;
      m_nHelpProduction.RecognizedBegin := n.RecognizedBegin;
    end; //else
    m_nHelpProduction := m_nHelpProduction.Parent as TTetraNode;
  end;
  ttRelationTree :
  begin
    if m_pnlReturn <> NIL then
    begin
      if m_pnlHelpProduction.m_pFirstChild <> NIL then
      begin
        AddChildLast(m_pnlHelpProduction, m_pnlReturn);
        m_pnlReturn := NIL;
      end;
    end;

    if m_pnlHelpProduction.m_pFirstChild <> NIL then
    begin
      pnReturn := m_pnlHelpProduction;
      p := m_pnlHelpProduction.m_pFirstChild;
      m_pnlHelpProduction.m_This.SymbolNumber := p^.m_This.Symno;
      m_pnlHelpProduction.m_This.IgnoredBegin := p^.m_This.IgnoredBegin;
      m_pnlHelpProduction.m_This.RecognizedBegin := p^.m_This.RecognizedBegin;
      while p^.m_pNextSibling <> NIL do
        p := p^.m_pNextSibling;
      m_pnlHelpProduction.m_This.RecognizedEnd := p^.m_This.RecognizedEnd;
    end
    else
      pnReturn := m_pnlReturn;

    m_ProductionList.Remove(m_ProductionList.Last());
    if m_ProductionList.Count > 0 then
    begin
      m_pnlHelpProduction := m_ProductionList.Last();

      if m_pnlHelpProduction.m_pFirstChild <> NIL then
      begin
         AddChildLast(m_pnlHelpProduction, pnReturn);
         m_pnlReturn := NIL;
      end
      else
         m_pnlReturn := pnReturn;
    end
    else
    begin
      m_Tree.Items.AddNode(m_pnlHelpProduction.m_This, NIL, m_pnlHelpProduction.m_This.Text, nil, naAddChild);
      if pnReturn <> m_pnlHelpProduction then
         m_pnlHelpProduction.m_pFirstChild := pnReturn;
      InsertChildren(m_pnlHelpProduction);
    end;
  end;
  end;
end;

procedure TTextTreeViewTransformer.InsertChildren(node : PTetraNodeLink);
var
  cur : PTetraNodeLink;
begin
  cur := node.m_pFirstChild;
  while cur <> NIL do
  begin
    m_Tree.Items.AddNode(cur.m_This, node.m_This, cur.m_This.Text, nil, naAddChild);
    InsertChildren(cur);
    cur := cur.m_pNextSibling;
  end;

end;


procedure TTextTreeViewTransformer.BeforeBeginBranch(const xasName: string; xeNodeKind : eTetraNodeKind);
begin
(** add new node as last child of m_nHelpProduction (root node if nil).
  Make the new node the new m_nHelpProduction.
*)
  if (m_eTreeType <> ttFullTree) or (m_Tree = NIL) then
     EXIT;

  if m_eTreeType = ttFullTree then
    m_nHelpProduction := AddNode(xeNodeKind, xasName);

end;

procedure TTextTreeViewTransformer.BeforeEndBranch();
var
  n: TTetraNode;
begin
(** Revert m_nHelpProduction to its parent.
  (all child nodes have been added)

  What if no children? The positions stay zero, may invalidate the parent's positions!
*)
  if (m_eTreeType <> ttFullTree) or (m_Tree = NIL) then
     EXIT;

  n := m_nHelpProduction.GetLastChild as TTetraNode;
  if n <> nil then begin
  //has children, get the total range
    m_nHelpProduction.RecognizedEnd := n.RecognizedEnd;
    n := m_nHelpProduction.GetFirstChild as TTetraNode;
    m_nHelpProduction.SymbolNumber := n.Symno;
    m_nHelpProduction.IgnoredBegin := n.IgnoredBegin;
    m_nHelpProduction.RecognizedBegin := n.RecognizedBegin;
  end; //else
  m_nHelpProduction := m_nHelpProduction.Parent as TTetraNode;
end;


{ TTetraNode }

function TTetraNode.IgnoredText: AnsiString;
begin
  Result := MidStr(self.SourceText, IgnoredBegin, RecognizedBegin - IgnoredBegin);
end;

function TTetraNode.RecognizedText: AnsiString;
begin
  Result := MidStr(self.SourceText, RecognizedBegin, RecognizedEnd - RecognizedBegin);
end;

function TTetraNode.Symno: Integer;
begin
  Result := SymbolNumber;
end;

end.
